/// <reference types="node" />
declare module '@ioc:Adonis/Core/Server' {
    import { Server as HttpsServer } from 'https';
    import { RequestConfigContract } from '@ioc:Adonis/Core/Request';
    import { ResponseConfigContract } from '@ioc:Adonis/Core/Response';
    import { HttpContextContract } from '@ioc:Adonis/Core/HttpContext';
    import { MiddlewareStoreContract } from '@ioc:Adonis/Core/Middleware';
    import { RouterContract } from '@ioc:Adonis/Core/Route';
    import { IncomingMessage, ServerResponse, Server as HttpServer } from 'http';
    export type HookNode = (ctx: HttpContextContract) => Promise<void>;
    export type ErrorHandlerNode = string | ((error: any, ctx: HttpContextContract) => Promise<any>);
    export type ResolvedErrorHandlerNode = {
        type: 'function';
        value: Exclude<ErrorHandlerNode, string>;
    } | {
        type: 'autoload' | 'binding';
        namespace: string;
        method: string;
    };
    export interface HooksContract {
        before(cb: HookNode): this;
        after(cb: HookNode): this;
    }
    export interface ServerContract {
        instance?: HttpServer | HttpsServer;
        router: RouterContract;
        hooks: HooksContract;
        middleware: MiddlewareStoreContract;
        errorHandler(handler: ErrorHandlerNode): this;
        handle(req: IncomingMessage, res: ServerResponse): Promise<void>;
        optimize(): void;
    }
    export type ServerConfigContract = RequestConfigContract & ResponseConfigContract;
    const Server: ServerContract;
    export default Server;
}
