/// <reference types="node" />
declare module '@ioc:Adonis/Core/Response' {
    import { CookieOptions } from '@poppinss/cookie';
    import { ServerResponse, IncomingMessage } from 'http';
    import { MacroableConstructorContract } from 'macroable';
    export type CastableHeader = string | number | boolean | string[] | number[] | boolean[];
    export type ResponseContentType = 'text/html' | 'text/plain' | 'application/octet-stream' | 'application/json' | 'unknown' | 'null';
    export type ResponseStream = NodeJS.ReadStream | NodeJS.ReadWriteStream | NodeJS.ReadableStream;
    export type LazyBody = {
        writer: any;
        args: any[];
    };
    export interface ResponseContract {
        hasLazyBody: boolean;
        lazyBody: LazyBody | null;
        finished: boolean;
        headersSent: boolean;
        isPending: boolean;
        request: IncomingMessage;
        response: ServerResponse;
        getHeader(key: string): string | string[] | number | undefined;
        header(key: string, value: CastableHeader): this;
        append(key: string, value: CastableHeader): this;
        safeHeader(key: string, value: CastableHeader): this;
        removeHeader(key: string): this;
        status(code: number): this;
        safeStatus(code: number): this;
        type(type: string, charset?: string): this;
        vary(field: string): this;
        setEtag(body: any, weak?: boolean): this;
        buildResponseBody(body: any): {
            body: any;
            type: ResponseContentType;
            originalType?: string;
        };
        send(body: any, generateEtag?: boolean): void;
        json(body: any, generateEtag?: boolean): void;
        jsonp(body: any, callbackName?: string, generateEtag?: boolean): void;
        stream(stream: ResponseStream, errorCallback?: ((error: NodeJS.ErrnoException) => any)): void;
        download(filePath: string, generateEtag?: boolean, errorCallback?: ((error: NodeJS.ErrnoException) => any)): void;
        attachment(filePath: string, name?: string, disposition?: string, generateEtag?: boolean, errorCallback?: ((error: NodeJS.ErrnoException) => any)): void;
        location(url: string): this;
        redirect(url: string, reflectQueryParams?: boolean, statusCode?: number): void;
        cookie(key: string, value: any, options?: Partial<CookieOptions>): this;
        plainCookie(key: string, value: any, options?: Partial<CookieOptions>): this;
        clearCookie(key: string): this;
        abort(body: any, status?: number): void;
        abortIf(conditional: any, body: any, status?: number): void;
        finish(): void;
    }
    export type ResponseConfigContract = {
        secret?: string;
        etag: boolean;
        jsonpCallbackName: string;
        cookie: Partial<CookieOptions>;
    };
    export interface ResponseConstructorContract extends MacroableConstructorContract {
        new (request: IncomingMessage, response: ServerResponse, config: ResponseConfigContract): ResponseContract;
    }
    const Response: ResponseConstructorContract;
    export default Response;
}
