declare module '@ioc:Adonis/Core/Route' {
    import { MacroableConstructorContract } from 'macroable';
    import { HttpContextContract } from '@ioc:Adonis/Core/HttpContext';
    import { MiddlewareNode, ResolvedMiddlewareNode } from '@ioc:Adonis/Core/Middleware';
    export type RouteHandlerNode = ((ctx: HttpContextContract) => Promise<any>) | string;
    export type ResolveRouteHandlerNode = {
        type: 'function';
        handler: Exclude<RouteHandlerNode, string>;
    } | {
        type: 'autoload' | 'binding';
        namespace: string;
        method: string;
    };
    export type RouteLookupNode = {
        handler: RouteHandlerNode;
        pattern: string;
        domain: string;
        name?: string;
    };
    export type RouteMatchers = {
        [param: string]: RegExp;
    };
    export type RouteNode = {
        pattern: string;
        handler: RouteHandlerNode;
        middleware: MiddlewareNode[];
        meta: {
            resolvedHandler?: ResolveRouteHandlerNode;
            resolvedMiddleware?: ResolvedMiddlewareNode[];
            namespace?: string;
        } & {
            [key: string]: any;
        };
        name?: string;
    };
    export type MethodNode = {
        tokens: any[];
        routes: {
            [pattern: string]: RouteNode;
        };
    };
    export type DomainNode = {
        [method: string]: MethodNode;
    };
    export type RoutesTree = {
        tokens: any[];
        domains: {
            [domain: string]: DomainNode;
        };
    };
    export type RouteDefinition = RouteNode & {
        methods: string[];
        domain?: string;
        matchers: RouteMatchers;
    };
    export type MatchedRoute = {
        route: RouteNode;
        params: any;
        subdomains: any;
    };
    export interface RouteContract {
        deleted: boolean;
        name: string;
        where(param: string, matcher: string | RegExp): this;
        prefix(prefix: string): this;
        domain(domain: string): this;
        middleware(middleware: any | any[], prepend?: boolean): this;
        as(name: string, append?: boolean): this;
        namespace(namespace: string): this;
        toJSON(): RouteDefinition;
    }
    export interface RouteResourceContract {
        routes: RouteContract[];
        only(names: string[]): this;
        except(names: string[]): this;
        apiOnly(): this;
        middleware(middleware: {
            [name: string]: any | any[];
        }): this;
        where(key: string, matcher: string | RegExp): this;
        namespace(namespace: string): this;
    }
    export interface RouteGroupContract {
        routes: (RouteContract | RouteResourceContract | BriskRouteContract | RouteGroupContract)[];
        where(param: string, matcher: RegExp | string): this;
        prefix(prefix: string): this;
        domain(domain: string): this;
        as(name: string): this;
        middleware(middleware: any | any[]): this;
        namespace(namespace: string): this;
    }
    export interface BriskRouteContract {
        route: RouteContract | null;
        setHandler(handler: any, invokedBy: string): RouteContract;
    }
    export type MakeUrlOptions = {
        qs?: any;
        params?: any;
        domainParams?: any;
        prefixDomain?: boolean;
    };
    export interface RouterContract {
        BriskRoute: MacroableConstructorContract;
        RouteGroup: MacroableConstructorContract;
        RouteResource: MacroableConstructorContract;
        Route: MacroableConstructorContract;
        routes: (RouteContract | RouteResourceContract | RouteGroupContract | BriskRouteContract)[];
        route(pattern: string, methods: string[], handler: RouteHandlerNode): RouteContract;
        any(pattern: string, handler: RouteHandlerNode): RouteContract;
        get(pattern: string, handler: RouteHandlerNode): RouteContract;
        post(pattern: string, handler: RouteHandlerNode): RouteContract;
        put(pattern: string, handler: RouteHandlerNode): RouteContract;
        patch(pattern: string, handler: RouteHandlerNode): RouteContract;
        destroy(pattern: string, handler: RouteHandlerNode): RouteContract;
        group(callback: () => void): RouteGroupContract;
        resource(resource: string, controller: string): RouteResourceContract;
        shallowResource(resource: string, controller: string): RouteResourceContract;
        on(pattern: string): BriskRouteContract;
        where(key: string, matcher: string | RegExp): this;
        toJSON(): RouteNode[];
        commit(): void;
        match(url: string, method: string, domain?: string): null | MatchedRoute;
        lookup(routeIdentifier: string, domain?: string): null | RouteLookupNode;
        forTesting(pattern?: string, methods?: string[], handler?: any): RouteContract;
        makeUrl(routeIdentifier: string, options?: MakeUrlOptions, domain?: string): string | null;
        makeSignedUrl(routeIdentifier: string, options?: MakeUrlOptions & {
            expiresIn?: string | number;
        }, domain?: string): string | null;
    }
    const Route: RouterContract;
    export default Route;
}
