"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pluralize_1 = require("pluralize");
const macroable_1 = require("macroable");
const Route_1 = require("./Route");
class RouteResource extends macroable_1.Macroable {
    constructor(_resource, _controller, _globalMatchers, _shallow = false) {
        super();
        this._resource = _resource;
        this._controller = _controller;
        this._globalMatchers = _globalMatchers;
        this._shallow = _shallow;
        this.routes = [];
        this._buildRoutes();
    }
    _makeRoute(pattern, methods, action, baseName) {
        const route = new Route_1.Route(pattern, methods, `${this._controller}.${action}`, this._globalMatchers);
        route.as(`${baseName}.${action}`);
        this.routes.push(route);
    }
    _buildRoutes() {
        const resourceTokens = this._resource.split('.');
        const mainResource = resourceTokens.pop();
        const baseUrl = `${resourceTokens
            .map((token) => `${token}/:${pluralize_1.singular(token)}_id`)
            .join('/')}/${mainResource}`;
        const memberBaseUrl = this._shallow ? mainResource : baseUrl;
        const baseName = this._shallow ? mainResource : this._resource;
        this._makeRoute(baseUrl, ['GET'], 'index', this._resource);
        this._makeRoute(`${baseUrl}/create`, ['GET'], 'create', this._resource);
        this._makeRoute(baseUrl, ['POST'], 'store', this._resource);
        this._makeRoute(`${memberBaseUrl}/:id`, ['GET'], 'show', baseName);
        this._makeRoute(`${memberBaseUrl}/:id/edit`, ['GET'], 'edit', baseName);
        this._makeRoute(`${memberBaseUrl}/:id`, ['PUT', 'PATCH'], 'update', baseName);
        this._makeRoute(`${memberBaseUrl}/:id`, ['DELETE'], 'destroy', baseName);
    }
    _filter(names, inverse) {
        return this.routes.filter((route) => {
            const match = names.find((name) => route.name.endsWith(name));
            return inverse ? !match : match;
        });
    }
    only(names) {
        this
            ._filter(names, true)
            .forEach((route) => (route.deleted = true));
        return this;
    }
    except(names) {
        this
            ._filter(names, false)
            .forEach((route) => (route.deleted = true));
        return this;
    }
    apiOnly() {
        return this.except(['.create', '.edit']);
    }
    middleware(middleware) {
        for (let name in middleware) {
            const route = this.routes.find((route) => route.name.endsWith(name));
            if (route) {
                route.middleware(middleware[name]);
            }
        }
        return this;
    }
    where(key, matcher) {
        this.routes.forEach((route) => {
            route.where(key, matcher);
        });
        return this;
    }
    namespace(namespace) {
        this.routes.forEach((route) => {
            route.namespace(namespace);
        });
        return this;
    }
}
exports.RouteResource = RouteResource;
RouteResource._macros = {};
RouteResource._getters = {};
