"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const matchit_1 = __importDefault(require("matchit"));
const lodash_1 = require("lodash");
const utils_1 = require("@poppinss/utils");
class Store {
    constructor() {
        this.tree = { tokens: [], domains: {} };
    }
    _getDomainNode(domain) {
        if (!this.tree.domains[domain]) {
            this.tree.tokens.push(matchit_1.default.parse(domain));
            this.tree.domains[domain] = {};
        }
        return this.tree.domains[domain];
    }
    _getMethodRoutes(domain, method) {
        const domainNode = this._getDomainNode(domain);
        if (!domainNode[method]) {
            domainNode[method] = { tokens: [], routes: {} };
        }
        return domainNode[method];
    }
    add(route) {
        const routeJSON = lodash_1.cloneDeep(lodash_1.pick(route, [
            'pattern',
            'handler',
            'meta',
            'middleware',
            'name',
        ]));
        route.methods.forEach((method) => {
            const methodRoutes = this._getMethodRoutes(route.domain || 'root', method);
            if (methodRoutes.routes[route.pattern]) {
                throw new utils_1.Exception(`Duplicate route \`${method}:${route.pattern}\``, 500, 'E_DUPLICATE_ROUTE');
            }
            methodRoutes.tokens.push(matchit_1.default.parse(route.pattern, route.matchers));
            methodRoutes.routes[route.pattern] = routeJSON;
        });
        return this;
    }
    match(url, method, domain) {
        const matchedDomain = matchit_1.default.match(domain || 'root', this.tree.tokens);
        if (!matchedDomain.length) {
            return null;
        }
        const matchedMethod = this.tree.domains[matchedDomain[0].old][method];
        if (!matchedMethod) {
            return null;
        }
        const matchedRoute = matchit_1.default.match(url, matchedMethod.tokens);
        if (!matchedRoute.length) {
            return null;
        }
        return {
            route: matchedMethod.routes[matchedRoute[0].old],
            params: matchit_1.default.exec(url, matchedRoute),
            subdomains: matchit_1.default.exec(domain || 'root', matchedDomain),
        };
    }
}
exports.Store = Store;
