"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const co_compose_1 = require("co-compose");
class RequestHandler {
    constructor(_middlewareStore, _router) {
        this._middlewareStore = _middlewareStore;
        this._router = _router;
    }
    async _invokeHandler(ctx) {
        await this
            ._globalMiddleware
            .runner()
            .resolve(this._middlewareStore.invokeMiddleware.bind(this._middlewareStore))
            .finalHandler(ctx.route.meta.finalHandler, [ctx])
            .run([ctx]);
    }
    _findRoute(ctx) {
        const url = ctx.request.url();
        const method = ctx.request.method();
        const matchRoute = ctx.profiler.profile('http:route:match');
        const route = this._router.match(url, method);
        matchRoute.end();
        ctx.response.abortIf(!route, `Cannot ${method}:${url}`, 404);
        ctx.params = route.params;
        ctx.subdomains = route.subdomains;
        ctx.route = route.route;
    }
    async handle(ctx) {
        this._findRoute(ctx);
        await this._invokeHandler(ctx);
    }
    commit() {
        const middleware = this._middlewareStore.get();
        if (middleware.length) {
            this._globalMiddleware = new co_compose_1.Middleware().register(middleware);
        }
        else {
            this._invokeHandler = async (ctx) => ctx.route.meta.finalHandler(ctx);
        }
    }
}
exports.RequestHandler = RequestHandler;
